import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

from sklearn import tree
from sklearn import metrics
from sklearn.ensemble import RandomForestClassifier
from sklearn.metrics import confusion_matrix
from sklearn.cross_validation import train_test_split
from sklearn import cross_validation
from sklearn.cross_validation import cross_val_score
from sklearn import neighbors
from sklearn import linear_model
from sklearn.naive_bayes import GaussianNB
from sklearn import svm


__author__ = 'Riccardo Guidotti'


def categorical_to_numeric(categories):
    category_to_number = dict()
    number_to_category = dict()

    index = 0
    for c in categories:
        category_to_number[c] = index
        number_to_category[index] = c

    return category_to_number, number_to_category


def main():

    input_filename = '/Users/riccardo/Documents/Didattica/CorsoDM_2015_2016/python-workspace/iris.csv'
    df = pd.read_csv(input_filename, skipinitialspace=True)

    # train test split
    x_col = df.columns.tolist()
    x_col.remove('class')
    train = df[x_col]
    test = df['class']

    train_x, test_x, train_y, real_y = train_test_split(train, test, test_size=0.2)

    # decision tree classifier
    # criterion gini, entropy

    print '\ndecision tree'
    clf = tree.DecisionTreeClassifier(criterion='gini', splitter='best', max_depth=None, min_samples_split=2,
                                      min_samples_leaf=2)
    clf = clf.fit(train_x, train_y)

    print '', clf.feature_importances_

    pred_y = clf.predict(test_x)

    print metrics.precision_score(real_y, pred_y, average='weighted')
    print metrics.recall_score(real_y, pred_y, average='weighted')
    print metrics.f1_score(real_y, pred_y, average='weighted')
    print metrics.accuracy_score(real_y, pred_y)
    print metrics.precision_recall_fscore_support(real_y, pred_y)
    # roc_auc_score(real_y, pred_y)

    cm = confusion_matrix(real_y, pred_y)
    print cm

    plt.matshow(cm)
    plt.title('Confusion matrix')
    plt.colorbar()
    plt.ylabel('True label')
    plt.xlabel('Predicted label')
    plt.show()

    # cross validation
    scores = cross_validation.cross_val_score(clf, train, test, cv=10)
    print 'Accuracy: %0.2f (+/- %0.2f)' % (scores.mean(), scores.std() * 2)

    # random forest classifier

    print '\nrandom forest'
    clf = RandomForestClassifier(n_estimators=10, max_depth=None, min_samples_split=1, random_state=0)
    scores = cross_val_score(clf, train_x, train_y)
    print scores.mean()

    # random forest classifier

    print '\nnearest neighbor'
    clf = neighbors.KNeighborsClassifier(15, weights='distance')
    clf.fit(train_x, train_y)
    scores = cross_val_score(clf, train_x, train_y)
    print scores.mean()

    # support vector machines

    print '\nsupport vector machines'
    clf = svm.SVC()
    clf.fit(train_x, train_y)
    scores = cross_val_score(clf, train_x, train_y)
    print scores.mean()

    # print clf.support_vectors_
    # print clf.support_
    # print clf.n_support_

    # gaussian naive bayes

    print '\ngaussian naive bayes'
    gnb = GaussianNB()
    pred_y = gnb.fit(train_x, train_y).predict(test_x)
    print 'Number of mislabeled points out of a total %d points : %d' % (len(real_y), (real_y != pred_y).sum())

    # linear regression

    print '\nlinear regression'
    train_y = train_y.map({'Iris-setosa': 0, 'Iris-versicolor': 1, 'Iris-virginica': 2}).astype(int)
    real_y = real_y.map({'Iris-setosa': 0, 'Iris-versicolor': 1, 'Iris-virginica': 2}).astype(int)
    regr = linear_model.LinearRegression()
    regr.fit(train_x, train_y)

    print 'Coefficients: \n', regr.coef_
    print 'Residual sum of squares: %.2f' % np.mean((regr.predict(test_x) - real_y) ** 2)
    print 'Variance score: %.2f' % regr.score(test_x, real_y)

if __name__ == "__main__":
    main()